
import bpy
import mathutils
import random

colRED = mathutils.Vector((1,0,0,1))
colGRN = mathutils.Vector((0,1,0,1))
colBLU = mathutils.Vector((0,0,1,1))
colON = mathutils.Vector((1,0,0,1))
colOFF = mathutils.Vector((0,0,0,1))
colBLK = mathutils.Vector((0,0,0,1))
colWHT = mathutils.Vector((1,1,1,1))

print('-----GO-----')

####################################
######## PIXEL ACCESS FXNS #########
####################################
def getpixel(img, x ,y):
    co = (y * img.size[0] + x) * 4
    ret = (
          img.pixels[co + 0],
          img.pixels[co + 1],
          img.pixels[co + 2],
          img.pixels[co + 3]            
          )
    return ret

def setpixel(img, x ,y, rgba):
    co = (y * img.size[0] + x) * 4
    img.pixels[co + 0] = rgba[0]
    img.pixels[co + 1] = rgba[1]
    img.pixels[co + 2] = rgba[2]
    img.pixels[co + 3] = rgba[3]

###################################
######## IMAGE SETUP FXNS #########
###################################
def randimg(img):
    for pxi in range(len(thisimg.pixels)):
        rint = random.random()
        thisimg.pixels[pxi] = rint

def blankimg(img):
    for pxi in range(len(thisimg.pixels)):
        thisimg.pixels[pxi] = 0.0

def con_blank(img):
    for y in range(img.size[1]):
        for x in range(img.size[0]):
            setpixel(img, x, y, colOFF)
    
def con_random(img, den):
    RANDOM_DENSITY = den
    for y in range(img.size[1]):
        for x in range(img.size[0]):
            rand = random.uniform(0.0, 1.0)
            if rand <= RANDOM_DENSITY:
                setpixel(img, x, y, colON)
            else:
                setpixel(img, x, y, colOFF)

def con_glider(img, x, y):
    #x = 20; y = 20
    Lglider = [(x,y), (x+1,y), (x+1,y+1), (x+2,y+1), (x,y+2)]
    for pix in Lglider: setpixel(img, pix[0], pix[1], colON)

def con_spaceship(img, x, y):    
    #x = 13; y = 13        
    Lspaceship = [(x,y), (x+3,y), (x+4,y+1), (x,y+2), (x+4,y+2),
                  (x+1,y+3), (x+2,y+3), (x+3,y+3), (x+4,y+3)]
    for pix in Lspaceship: setpixel(img, pix[0], pix[1], colON)

def con_glidergun(img, x, y):        
    #x = 5; y = 20    
    Lglidergun = [(x,y+3), (x,y+4), (x+1,y+3), (x+1,y+4),
                  
                  (x+12,y), (x+13,y), (x+11,y+1), (x+15,y+1),
                  (x+10,y+2), (x+16,y+2), 
                  (x+10,y+3), (x+14,y+3), (x+16,y+3), (x+17,y+3),
                  (x+10,y+4), (x+16,y+4),
                  (x+11,y+5), (x+15,y+5), (x+12,y+6), (x+13,y+6),
                  
                  (x+20,y+4), (x+20,y+5), (x+20,y+6),
                  (x+21,y+4), (x+21,y+5), (x+21,y+6),
                  (x+22,y+3), (x+22,y+7),
                  (x+24,y+2), (x+24,y+3), (x+24,y+7), (x+24,y+8),
                  
                  (x+34,y+5), (x+34,y+6), (x+35,y+5), (x+35,y+6)]

    for pix in Lglidergun: setpixel(img, pix[0], pix[1], colON)

####################################
########## CELL AUTO FXNS ##########
####################################

def getNeighbors(x,y, wd, ht):
    nL = []
    nL.append(((x-1)%wd, (y-1)%ht))
    nL.append(((x-1)%wd, y))
    nL.append(((x-1)%wd, (y+1)%ht))
    nL.append(((x, (y-1)%ht)))
    nL.append(((x, (y+1)%ht)))
    nL.append(((x+1)%wd, (y-1)%ht))
    nL.append(((x+1)%wd, y))
    nL.append(((x+1)%wd, (y+1)%ht))    
    return nL

def conwaysSimStep(img):
    PMIN = 2; PMAX = 3
    iwd = img.size[0] 
    iht = img.size[1] 

    ###---LOAD OUTPUT IMAGE
    if 'tempimg' in bpy.data.images:
        imgOUT = bpy.data.images['tempimg']
    else: imgOUT = bpy.data.images.new('tempimg', iwd, iht)
    con_blank(imgOUT)
            
    for y in range(iht):
        for x in range(iwd):
            cAlive = round(getpixel(img, x, y)[0]) ###RED CHANNEL IS LIVE/DEAD
            #cAlive = cPix 
            cAliveNext = 0
            cNAliveTot = 0
            cNeighbors = getNeighbors(x, y, iwd, iht)
            for c in cNeighbors:
                cNAliveTot += round(getpixel(img, c[0], c[1])[0])
            ###---CONWAYS LIFE RULES
            if cAlive == 1 and cNAliveTot < PMIN:
                cAliveNext = 0        #-death|underpop
            elif cAlive == 1 and cNAliveTot >= PMIN and cNAliveTot <= PMAX:            
                cAliveNext = 1        #-lives
            elif cAlive == 1 and cNAliveTot > PMAX:
                cAliveNext = 0        #-death|overpop
            elif cAlive == 0 and cNAliveTot == PMAX:
                cAliveNext = 1        #-birth
            #print(x,y, '-', cAlive, cNAliveTot, cAliveNext)
            ###---UPDATE CELL
            if cAlive != cAliveNext:
                if cAliveNext == 0:
                    setpixel(imgOUT, x, y, colOFF)
                elif cAliveNext > 0:
                    setpixel(imgOUT, x, y, colON)

    ###---ADD IN RED CHANNEL FROM NEXT DYN PAINT IMAGE HERE

    return imgOUT


def conwaySimLoop(icachepath, icachefile, iter):
    ###---SPLIT NAME, NUM, AND TYPE
    fname = icachefile[:-8]
    fnum = icachefile[-8:-4]
    ftype = icachefile[-4:]
    
    ###---LOAD INITIAL IMAGE
    if fname in bpy.data.images:
        imgIn = bpy.data.images[fname]
        imgIn.source = 'FILE'
        imgIn.filepath = icachepath + icachefile
        imgIn.reload()
    else: imgIn = bpy.data.images.load(icachepath + icachefile)

    #con_random(imgIn, .15)
    con_blank(imgIn)
    con_glidergun(imgIn, 5, 18)
    con_glider(imgIn, 8, 8)
    imgIn.save_render(icachepath + 'clife/' + icachefile)
    
    
    ###---LOAD DYNAMIC PAINT IMAGE
    if 'dpaintimg' in bpy.data.images:
        imgDPaint = bpy.data.images['dpaintimg']
    else: imgDPaint = bpy.data.images.new('dpaintimg', imgIn.size[0], imgIn.size[1])

    ###---SIM LOOP     
    for n in range( int(fnum), iter+int(fnum) ):
        print(':running simstep -', n, 'of', iter+int(fnum))
        
        ###---OUTPUT PATH AND NEXT DYN PAINT SIM IMAGE
        thisnumstr = str(10000 + n)[-4:]
        thispath = icachepath + 'clife/' + fname + thisnumstr + ftype        
        nextnumstr = str(10000 + n + 1)[-4:]
        nextpath = icachepath + fname + nextnumstr + ftype
        outputpath = icachepath + 'clife/' + fname + nextnumstr + ftype        

        ###---SIM STEP
        imgOut = conwaysSimStep(imgIn)
        
        ###---ADD PIXELS (RED) FROM DYN PAINT SIM TO OUTPUT
        imgDPaint.source = 'FILE'
        imgDPaint.filepath = nextpath
        imgDPaint.reload()
        for py in range(imgIn.size[0]):
            for px in range(imgIn.size[1]):
                if getpixel(imgDPaint, px, py)[0] >= 1:
                    setpixel(imgOut, px, py, colON)
                
        ###---SAVE OUTPUT FILE
        imgOut.save_render(outputpath)
        print(':saved output image - ', outputpath)
        
        ###---NEXT IMAGE
        imgIn = imgOut

        
####################################
####################################
####################################


tpath = "/home/t/Desktop/blendFiles/cache_dynamicpaint/"
#tpath = "/home/t/Desktop/blendFiles/"
#tpath = "\\home\\blends\\cache_dynamicpaint\\"
#tpath = "\\home\\blends\\"
tfile = "paintmap0001.png"
#tfile = "000testoutput.png"
count = 30



#con_random(imgstart, .5)

conwaySimLoop(tpath, tfile, count)
#print(round(.5))  
    
print('-----DONE-----')



